#-----------------------------------------------------------------------
# Canthology: software that makes it easy to create anthologies.
#
# Copyright (C) 2011 Ciaran McHale.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------

proc readFile {fileName} {
	if {[catch {
		set fd [open $fileName "r"]
		set contents [read $fd]
		close $fd
	} err]} {
		puts stderr $err
		exit 1
	}
	return $contents
}

proc writeFile {fileName contents} {
	if {[catch {
		set fd [open $fileName "w"]
		puts -nonewline $fd $contents
		close $fd
	} err]} {
		puts stderr $err
		exit 1
	}
}


proc findList {contents currIndex _startIndex _endIndex} {
	upvar	$_startIndex	startIndex
	upvar	$_endIndex	endIndex

	set endIndex -1
	set startIndex [string first "<UL>" $contents $currIndex]
	if {$startIndex == -1} {
		return 0; # list not found
	}
	set eofIndex [string length $contents]
	set currIndex [expr $startIndex + 4]
	set listDepth 1
	while {$listDepth != 0} {
		set i1 [string first "<UL>" $contents $currIndex]
		set i2 [string first "</UL>" $contents $currIndex]
		if {$i1 == -1 && $i2 == -1} {
			set startIndex -1
			return 0; # list not found
		}
		if {$i1 == -1} { set i1 $eofIndex }
		if {$i2 == -1} { set i2 $eofIndex }
		if {$i1 < $i2} {
			# Found <UL>
			incr listDepth
			set currIndex [expr $i1 + 4]
		} else {
			# Found </UL>
			incr listDepth -1
			set currIndex [expr $i2 + 5]
		}
	}
	set endIndex [expr $currIndex - 1]
	return 1; # list found
}


proc processHtmlFile {linkText contentsText fileName} {
	#--------
	# If a pseudo table of contents is present, then it
	# will be the last <UL> list in the file, and it will
	# have a hyperlink to "Contents"
	#
	# First step: locate the last <UL> list in the file
	#--------
	set contents [readFile $fileName]
	if {![findList $contents 0 startIndex endIndex]} {
		puts stderr "Cannot find '$contentsText' in $fileName"
		exit 1
	}
	set currIndex [expr $endIndex + 1]
	while {[findList $contents $currIndex tmpStartIndex tmpEndIndex]} {
		startIndex = $tmpStartIndex
		endIndex = $tmpEndIndex
		set currIndex [expr $endIndex + 1]
	}

	#--------
	# Second step: look for a link to $contentsText (usually "Contents")
	# within the list. If a \tableofcontents command had not been used,
	# then there won't be such a link.
	#--------
	set toc [string range $contents $startIndex $endIndex]
	set pattern ""; append pattern {<A HREF="([^"]+)">} "$contentsText</A>"
	set found [regexp $pattern $toc matchedText contentsFile]
	if {!$found} {
		set linkText ""
	}

	#--------
	# Final step: Remove the pseudo table of contents list, and
	# optionally replace it with a link to the real table of contents.
	#--------
	if {$linkText == ""} {
		set linkToToc ""
	} else {
		set linkToToc "<A HREF=\"$contentsFile\">$linkText</A>"
	}
	set newContents [string replace $contents $startIndex $endIndex \
	                 $linkToToc]
	writeFile $fileName $newContents
}


proc usage {} {
	set prog [file tail $::argv0]
	set requiredArgs "-contentsname <text> <file.html>"
	puts stderr "usage: tclsh $prog -link <link text> $requiredArgs"
	puts stderr "or:    tclsh $prog -nolink $requiredArgs"
	exit 1
}


#--------
# Mainline
#--------
if {$argc != 4 && $argc != 5} {
	usage 
}
set option [lindex $argv 0]
if {$argc == 4} {
	foreach {noLinkOpt cnOpt contentsText fileName} $argv {}
	if {$noLinkOpt != "-nolink"} { usage }
	if {$cnOpt != "-contentsname"} { usage }
	processHtmlFile "" $contentsText $fileName
} else {
	foreach {linkOpt linkText cnOpt contentsText fileName} $argv {}
	if {$linkOpt != "-link"} { usage }
	if {$cnOpt != "-contentsname"} { usage }
	processHtmlFile $linkText $contentsText $fileName
}

